<?php

// Copyright 2007-2009 Jonathan Brown


define("OP_VIDEO_OPENPACKAGE_URL", 'http://openpackage.biz');

define("OP_VIDEO_DEFAULT_OPENPACKAGE_RETURN_METHOD", 'poll');
define("OP_VIDEO_OPENPACKAGE_PROTO_VERSION", 4);

define("HTTP_CONNECT_TIMEOUT", 4);
define("HTTP_LOW_SPEED_LIMIT", 256);
define("HTTP_LOW_SPEED_TIMEOUT", 600);


function _op_video_openpackage_settings_fieldset($active) {

  $fieldset = array(
    '#type' => 'fieldset',
    '#title' => t('OpenPackage Media'),
    '#collapsible' => TRUE,
    '#collapsed' => !$active
  );

  $fieldset['op_video_openpackage_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#size' => 15,
    '#maxlength' => USERNAME_MAX_LENGTH,
    '#default_value' => variable_get('op_video_openpackage_name', '')
  );

  $fieldset['op_video_openpackage_hash'] = array(
    '#type' => 'password',
    '#title' => t('Password'),
    '#maxlength' => 60,
    '#size' => 15
  );

  $description  = 'Direct return is the optimal setting, but will not work if your site is not visible on the ';
  $description .= 'Internet.<br />If you are not running cron.php through your web server, ensure that it is running ';
  $description .= 'as the same user as your web server and that you have defined $base_url in settings.php.';
  
  $fieldset['op_video_openpackage_return_method'] = array(
    '#type' => 'radios',
    '#title' => t('Return method'),
    '#options' => array(
      'direct' => 'Direct return',
      'poll' => 'Periodic check'
    ),
    '#default_value' => 
      variable_get('op_video_openpackage_return_method', OP_VIDEO_DEFAULT_OPENPACKAGE_RETURN_METHOD),
    '#description' => t($description)
  );

  return $fieldset;
}


function _op_video_reattempt_invalid_credentials() {

  db_query("
    UPDATE {op_videos}
    SET  status = 'dirty',
      error = 'none',
      upload_failed = 0
    WHERE error = 'invalid credentials'
  ");

  if(db_affected_rows()) {
    $msg = 'Reattempting uploads that failed due to invalid credentials.';
    drupal_set_message($msg);
    watchdog('op_video', $msg, NULL);
  }
}


function _op_video_openpackage_settings_form_submit(&$form_state) {

  if($form_state['clicked_button']['#value'] != 'Save configuration')
    return;

  $form_state['values']['op_video_openpackage_hash'] = trim($form_state['values']['op_video_openpackage_hash']);

  if($form_state['values']['op_video_transcoder'] == 'openpackage.biz' &&
    $form_state['values']['op_video_openpackage_hash'] != '')
  {
    $form_state['values']['op_video_openpackage_hash'] = md5($form_state['values']['op_video_openpackage_hash']);
    _op_video_reattempt_invalid_credentials();
  }
  else {
    unset($form_state['values']['op_video_openpackage_hash']);
  }
}


function _op_video_curl_post($url, $fields, &$output) {

  $options = array(
    CURLOPT_POST => TRUE,
    CURLOPT_POSTFIELDS => $fields,
    CURLOPT_CONNECTTIMEOUT => HTTP_CONNECT_TIMEOUT,
    CURLOPT_LOW_SPEED_LIMIT => HTTP_LOW_SPEED_LIMIT,
    CURLOPT_LOW_SPEED_TIME => HTTP_LOW_SPEED_TIMEOUT,
    CURLOPT_RETURNTRANSFER => TRUE
  );
  
  $ch = curl_init($url);
  curl_setopt_array($ch, $options);
  $output = curl_exec($ch);
  $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
  curl_close($ch);
  
  return $http_code;
}


function _op_video_upload_video($video) {
  
  $fields['proto_ver'] = OP_VIDEO_OPENPACKAGE_PROTO_VERSION;

  $fields['name'] = variable_get('op_video_openpackage_name', '');
  $fields['hash'] = variable_get('op_video_openpackage_hash', '');

  $fields['site_url'] = url('', array('absolute' => TRUE));
  $fields['return_method'] = variable_get('op_video_openpackage_return_method', 
    OP_VIDEO_DEFAULT_OPENPACKAGE_RETURN_METHOD);
  $fields['video_id'] = $video['video_id'];
  $fields['secret'] = $video['secret'];

  unset($video['dirty_params']['keep_source_file']);
  $fields['params'] = serialize($video['dirty_params']);

  $filepath = storage_api_get_local_filepath($video['source_file'], $keep_me);
  $fields['video'] = '@' . $filepath;
  $fields['filename'] = $video['source_file']['filename'];

  $url = variable_get('op_video_openpackage_url', OP_VIDEO_OPENPACKAGE_URL) . '/incoming';
  
  $http_code = _op_video_curl_post($url, $fields, $output);
  
  if(!$keep_me)
    @unlink($filepath);
  
  _op_video_reset_cron();

  return $http_code;
}


function _op_video_transcode_openpackage($video) {

/*
  if($video['upload_failed'] && 
    $video['upload_failed'] > time() - 10 * 60)
  {
    return;
  }
*/

  $http_code = _op_video_upload_video($video);
  
  if(!$http_code) {

    db_query("
      UPDATE {op_videos}
      SET status = 'idle',
        error = 'file not found'
      WHERE video_id = %d
    ",
      $video['video_id']
    );
  }
  else {
    switch($http_code) {

      case 200:
        db_query("
          UPDATE {op_videos}
          SET status = 'uploaded',
            error = 'none',
            uploaded = %d
          WHERE video_id = %d
        ",
          time(),
          $video['video_id']
        );

        watchdog('op_video', 'Video successfully uploaded. video_id: ' . $video['video_id'], NULL);
        break;

      case 402:
        db_query("
          UPDATE {op_videos}
          SET status = 'uploaded',
            error = 'insufficient credit',
            uploaded = %d
          WHERE video_id = %d
        ",
          time(),
          $video['video_id']
        );

        watchdog('op_video', 'Video uploaded. Insufficient credit. video_id: ' . $video['video_id'], NULL, WATCHDOG_WARNING);
        break;

      case 403:
        db_query("
          UPDATE {op_videos}
          SET status = 'idle',
            error = 'invalid credentials'
          WHERE video_id = %d
        ",
          $video['video_id']
        );

        watchdog('op_video', 'Video upload failed. Invalid credentials. video_id: ' . $video['video_id'], NULL, WATCHDOG_WARNING);
        break;

      case 406:
        _op_video_set_transcoding_failed($video['video_id']);
//        watchdog('op_video', 'Video upload failed. Invalid credentials. video_id: ' . $video['video_id'], NULL, WATCHDOG_WARNING);
        break;

      default:
        db_query("
          UPDATE {op_videos}
          SET status = 'dirty',
            error = 'connection error',
            upload_failed = %d
          WHERE video_id = %d
        ",
          time(),
          $video['video_id']
        );

        watchdog('op_video', 'Video upload failed - will reattempt in 10 min. Connection error. video_id: ' . $video['video_id'], NULL, WATCHDOG_WARNING);
        break;
    }
  }
}


function _op_video_curl_post_get_file($url, $fields, $filepath) {

  $fp = fopen($filepath, "w");

  $options = array(
    CURLOPT_POST => TRUE,
    CURLOPT_POSTFIELDS => $fields,
    CURLOPT_CONNECTTIMEOUT => HTTP_CONNECT_TIMEOUT,
    CURLOPT_LOW_SPEED_LIMIT => HTTP_LOW_SPEED_LIMIT,
    CURLOPT_LOW_SPEED_TIME => HTTP_LOW_SPEED_TIMEOUT,
    CURLOPT_FILE => $fp
  );
  
  $ch = curl_init($url);
  curl_setopt_array($ch, $options);
  curl_exec($ch);
  fclose ($fp);
  $result = curl_getinfo($ch, CURLINFO_HTTP_CODE);
  curl_close($ch);
  
  _op_video_reset_cron();

  return $result;
}


function _op_video_openpackage_get_file($video, $file) {
  $url = variable_get('op_video_openpackage_url', OP_VIDEO_OPENPACKAGE_URL) . '/get';

  $fields['name'] = variable_get('op_video_openpackage_name', '');
  $fields['hash'] = variable_get('op_video_openpackage_hash', '');
  $fields['site_url'] = url('', array('absolute' => TRUE));
  $fields['video_id'] = $video['video_id'];
  $fields['file'] = $file;

  $selector_ids = _op_video_get_selector_ids($video['field']);

  $filepath = tempnam(file_directory_temp(), '');
  
  $result = _op_video_curl_post_get_file($url, $fields, $filepath);
  
  if($result == 200) {
  
    $options = array(
      'module' => 'op_video'
    );
    
    switch($file) {
      
      case 'flv':
        $options['type'] = 'transcoded video';
        break;
  
      case 'raw_image_1':
      case 'raw_image_2':
        $options['type'] = 'preview image';
        break;
    }
        
    return storage_api_add_file_from_filepath($filepath, $selector_ids['transcoded'], $options);
  }
  else {
    @unlink($filepath);
    return -$result;
  }
}


function _op_video_openpackage_get_info($video) {
  $url = variable_get('op_video_openpackage_url', OP_VIDEO_OPENPACKAGE_URL) . '/get';

  $fields['name'] = variable_get('op_video_openpackage_name', '');
  $fields['hash'] = variable_get('op_video_openpackage_hash', '');
  $fields['site_url'] = url('', array('absolute' => TRUE));
  $fields['video_id'] = $video['video_id'];
  $fields['info'] = TRUE;

  _op_video_curl_post($url, $fields, $output);
  
  return unserialize($output);
}


function _op_video_retrieve_videos() {

  $result = db_query("
    SELECT video_id
    FROM {op_videos}
    WHERE status = 'uploaded'
    ORDER BY video_id
  ");

  while($video = db_fetch_array($result)) {
    $video = _op_video_load($video['video_id']);
  
    $flv_file_id = _op_video_openpackage_get_file($video, 'flv');
    
    if($flv_file_id > 0) {
      $raw_image_1_file_id = _op_video_openpackage_get_file($video, 'raw_image_1');
      $raw_image_2_file_id = _op_video_openpackage_get_file($video, 'raw_image_2');
      $info = _op_video_openpackage_get_info($video);

      $params_id = _op_video_transcoding_params_insert($info['params']);

      db_query("
        UPDATE {op_videos}
        SET duration = %d,
          flv_file_id = %d,
          start_image_file_id = %d,
          third_image_file_id = %d,
          current_params_id = %d,
          status = 'idle',
          error = 'none'
        WHERE video_id = %d
      ",
        $info['duration'],
        $flv_file_id,
        $raw_image_1_file_id,
        $raw_image_2_file_id,
        $params_id,
        $video['video_id']
      );
      
      storage_api_inherit_servings($flv_file_id, $video['flv_file_id']);
      
      storage_api_remove_file($video['flv_file_id']);
      storage_api_remove_file($video['start_image_file_id']);
      storage_api_remove_file($video['third_image_file_id']);

      _op_video_transcoding_completed($video['video_id']);
      watchdog('op_video', 'Video returned. video_id: ' . $video['video_id'], NULL);
    }
    else {
    
      switch(-$flv_file_id) {

        case 204:    // not transcoded yet
          break;
      
        case 403:
          db_query("
            UPDATE {op_videos}
            SET error = 'invalid credentials'
            WHERE video_id = %d
          ",
            $video['video_id']
          );
          break;
      
        case 406:
          _op_video_set_transcoding_failed($video['video_id']);
          break;
      }
    }
  }
}


function _op_video_incoming() {

  watchdog('op_video', var_export($_POST, TRUE), NULL);

  $video_id = (int)$_POST['video_id'];

  $video = db_fetch_array(db_query("
    SELECT *
    FROM {op_videos}
    WHERE video_id = %d AND
      secret = '%s'
  ",
    $video_id,
    $_POST['secret']
  ));

  if(!$video) {
    watchdog('op_video', 'Returned video invalid. $_POST: ' . var_export($_POST, TRUE), NULL, WATCHDOG_WARNING);
    drupal_set_header('HTTP/1.0 200 OK');
    exit();
  }

  if($_POST['error']) {
    _op_video_set_transcoding_failed($video_id);
    watchdog('op_video', 'Error returned. video_id: ' . $video_id, NULL);
    drupal_set_header('HTTP/1.0 200 OK');
    exit();
  }

  $params = unserialize($_POST['params']);
  $params_id = _op_video_transcoding_params_insert($params);

  foreach($_FILES as $source_key => $source_value)
    foreach($source_value as $key => $value)
      $_FILES['files'][$key][$source_key] = $value;

  $selector_ids = _op_video_get_selector_ids($video['field']);
  
  $options = array(
    'module' => 'op_video',
    'type' => 'preview image'
  );

  $start_image_file_id = storage_api_add_file_from_upload('raw_image_1', $selector_ids['transcoded'], $options);
  $third_image_file_id = storage_api_add_file_from_upload('raw_image_2', $selector_ids['transcoded'], $options);
  
  $options['type'] = 'transcoded video';

  $flv_file_id = storage_api_add_file_from_upload('flv', $selector_ids['transcoded'], $options);
    
  db_query("
    UPDATE {op_videos}
    SET duration = %d,
      flv_file_id = %d,
      start_image_file_id = %d,
      third_image_file_id = %d,
      current_params_id = %d,
      status = 'idle',
      error = 'none'
    WHERE video_id = %d
  ",
    $_POST['duration'],
    $flv_file_id,
    $start_image_file_id,
    $third_image_file_id,
    $params_id,
    $video_id
  );

  storage_api_inherit_servings($flv_file_id, $video['flv_file_id']);

  storage_api_remove_file($video['flv_file_id']);
  storage_api_remove_file($video['start_image_file_id']);
  storage_api_remove_file($video['third_image_file_id']);

  _op_video_transcoding_completed($video['video_id']);
  watchdog('op_video', 'Video returned. video_id: ' . $video_id, NULL);
  drupal_set_header('HTTP/1.0 200 OK');
  exit();
}

